@@ -25,16 +25,24 @@
 #ifndef GCHEMPAINT_VIEW_H
 #define GCHEMPAINT_VIEW_H
 
-#include <map>
-#include <list>
-#include <libgnomecanvas/libgnomecanvas.h>
-#include <canvas/gcp-canvas-pango.h>
 #include <gcu/macros.h>
-#include "atom.h"
-#include "bond.h"
+#include <gccv/client.h>
+#include <list>
+#include <map>
+
+namespace gccv {
+	class Canvas;
+	class Text;
+}
+
+namespace gcu {
+	class Object;
+};
 
 namespace gcp {
 
+class Atom;
+class Bond;
 class Document;
 class WidgetData;
 
@@ -44,7 +52,7 @@ extern GtkTargetEntry const targets[];
 /*!\class View gcp/view.h
 The GChempaint document view.
 */
-class View
+class View: public gccv::Client
 {
 public:
 	//Constructor and destructor
@@ -72,26 +80,17 @@ public:
 */
 	Document* GetDoc () {return m_pDoc;}
 /*!
-@param item the canvas item at the positionof the event.
-@param event the GdkEvent.
-@param widget the canvas widget.
-
-Called by the framework when an event occured in the canvas.
-@return false if the event should be processed further, true otherwise.
-*/
-	bool OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget);
-/*!
 @param pObject the object to add.
 
 Adds the object to the canvas.
 */
-	void AddObject (gcu::Object const *pObject);
+	void AddObject (gcu::Object *pObject);
 /*!
 @param pObject the object to update.
 
 Updates the object in the canvas.
 */
-	void Update (gcu::Object const *pObject);
+	void Update (gcu::Object *pObject);
 /*!
 Creates a new canvas widget for the view.
 
@@ -105,18 +104,6 @@ Called by the framework when a widget is destroyed.
 */
 	void OnDestroy (GtkWidget* widget);
 /*!
-@param widget the canvas in which an item is searched.
-@param object the object represented by the item.
-
-Searches the item representing \a object in the canvas widget.
-@return the found item if any, or NULL.
-*/
-	GnomeCanvasItem* GetCanvasItem (GtkWidget* widget, gcu::Object* object);
-/*!
-@return the canvas background.
-*/
-	GnomeCanvasItem* GetBackground ();
-/*!
 @return the current zoom factor.
 */
 	double GetZoomFactor ();
@@ -131,10 +118,6 @@ Removes the object from the view and destroys the items representing it.
 */
 	void Remove (gcu::Object* pObject);
 /*!
-@return the pango context associated to the view.
-*/
-	PangoContext* GetPangoContext () {return m_PangoContext;}
-/*!
 @return the height of the font used to display atomic symbols.
 */
 	double GetFontHeight () {return m_dFontHeight;}
@@ -198,31 +181,12 @@ Called by the framework when a key has been released on the keyboard.
 */
 	bool OnKeyRelease (GtkWidget* w, GdkEventKey* event);
 /*!
-@param w the widget which received the event.
-@param width the ne widget width.
-@param height the new widget height.
-
-Called by the framework when the widget size changed.
-@return true (always).
-*/
-	bool OnSize (GtkWidget *w, int width, int height);
-/*!
-@param x1 the left coordinate of the bounding rectangle of the used canvas area.
-@param y1 the top coordinate of the bounding rectangle of the used canvas area.
-@param x2 the right coordinate of the bounding rectangle of the used canvas area.
-@param y2 the bottom coordinate of the bounding rectangle of the used canvas area.
-
-Ensures the canvas is large enough to display everything and, if x1 or y1 is
-negative moves all the items so that they are visible.
-*/
-	void UpdateSize (double x1, double y1, double x2, double y2);
-/*!
 @param item a text item or NULL.
 
 Sets the currently edited text item. \a item should be NULL to tell the view
 that no text edition is currently taking place.
 */
-	void SetGnomeCanvasPangoActive (GnomeCanvasPango* item);
+	void SetTextActive (gccv::Text* item);
 /*!
 Called by the framework when the active window changes to stop current edition
 and inhibit timer events.
@@ -261,9 +225,13 @@ will be used which will result as a one to one pixel export.
 */
 	void ExportImage (std::string const &filename, const char* type, int resolution = -1);
 /*!
-@return the xml document containig the svg output.
+@return the svg output as a newly allocated string. Call g_free when done with it.
 */
-	xmlDocPtr BuildSVG ();
+	char *BuildSVG ();
+/*!
+@return the eps output as a newly allocated string. Call g_free when done with it.
+*/
+	char *BuildEPS ();
 /*!
 @param resolution the resolution for the new image.
 
@@ -299,14 +267,31 @@ Called by the framework to update the default font size after a theme change.
 Renders the document using cairo.
 */
 	void Render (cairo_t *cr);
+/*!
+@param object the object which seection state should be changed.
+@param state the new selection state.
 
+Changes the selection state of \a object if it owns an item and of all its descendents.
+*/
+	void SetSelectionState (gcu::Object *object, int state);
+	// Signals
+	// there is no needd to document these since the documentation in gccv/client.h is appropriate.
+	bool OnButtonPressed (gccv::ItemClient *client, unsigned button, double x, double y, unsigned state);
+	bool OnButtonReleased (gccv::ItemClient *client, unsigned button, double x, double y, unsigned state);
+	bool OnDrag (gccv::ItemClient *client, double x, double y, unsigned state);
+	bool OnMotion (gccv::ItemClient *client, double x, double y, unsigned state);
+	bool OnLeaveNotify (unsigned state);
+
+/*!
+@return the WidgetData associated with the View widget.
+*/
+	WidgetData *GetData () {return m_pData;}
 	//Implementation
 private:
 	WidgetData* m_pData;
 	Document* m_pDoc;
 	GtkWidget* m_pWidget;
 	std::list<GtkWidget*> m_Widgets;
-	PangoContext* m_PangoContext;
 	PangoFontDescription* m_PangoFontDesc, *m_PangoSmallFontDesc;
 	double m_dFontHeight;
 	gchar* m_sFontName, *m_sSmallFontName;
@@ -316,6 +301,8 @@ private:
 	GtkUIManager *m_UIManager;
 	bool m_Dragging;
 	gcu::Object *m_CurObject;
+	Atom *m_CurAtom;
+	
 
 /*!\fn GetBaseLineOffset()
 @return the vertical offset for algnment of an atomic symbol. This value is half the height of the "C" character.
@@ -324,18 +311,17 @@ GCU_RO_PROP (double, BaseLineOffset)
 /*!\fn GetActiveRichText()
 @return the currently edited text item if any, or NULL.
 */
-GCU_RO_PROP (GnomeCanvasPango*, ActiveRichText)
+GCU_RO_PROP (gccv::Text *, ActiveRichText)
+/*!\fn GetCHeight()
+@return the half height of a carbon atom symbol in the current theme.
+*/
+GCU_RO_PROP (double, CHeight)
+/*!\fn GetHWidth()
+@return the half width of a hydrogen atom symbol in the current theme.
+*/
+GCU_RO_PROP (double, HWidth)
 };
 
-/*!
-@param item the item for which the event occured.
-@param event the current GdkEvent.
-@param widget the canvas owning the item.
-
-A callback for events occuring in the canvas. Calls View::OnEvent() for the
-view owning \a widget.
-*/
-bool on_event (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget);
 /*!
 @param clipboard a GtkClipboard.
 @param selection_data the data to paste.
