@@ -4,7 +4,7 @@
  * GChemPaint text plugin
  * texttool.cc 
  *
- * Copyright (C) 2002-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,6 +25,8 @@
 #include "config.h"
 #include "texttool.h"
 #include <gcp/application.h>
+#include <gcp/atom.h>
+#include <gcp/bond.h>
 #include <gcp/document.h>
 #include <gcp/fragment.h>
 #include <gcp/settings.h>
@@ -42,6 +44,10 @@
 
 using namespace gcu;
 extern GtkTargetEntry const text_targets[];
+namespace gcp {
+extern xmlDocPtr pXmlDoc;
+};
+
 
 GtkTargetEntry const text_targets[] = {
 	{(char *) GCHEMPAINT_ATOM_NAME,  0, gcp::GCP_CLIPBOARD_NATIVE},
@@ -60,7 +66,7 @@ gcpTextTool::gcpTextTool (gcp::Application* App, string Id):
 {
 	m_Active = NULL;
 	m_bUndo = true;
-	m_CurNode = m_InitNode = NULL;
+	m_CurNode = m_InitNode = m_GroupNode = NULL;
 	m_Strikethrough = gccv::TextDecorationNone;
 	m_FontDesc = NULL;
 	gcp::Theme *pTheme = gcp::TheThemeManager.GetTheme ("Default");
@@ -69,11 +75,12 @@ gcpTextTool::gcpTextTool (gcp::Application* App, string Id):
 	m_Weight = pTheme->GetFontWeight ();
 	m_Stretch = pTheme->GetFontStretch ();
 	m_Variant = pTheme->GetFontVariant ();
-	m_Size = pTheme->GetFontSize ();
+	m_Size = pTheme->GetTextFontSize ();
 	m_Underline = gccv::TextDecorationNone;
 	m_Rise = 0;
 	m_Color = GO_COLOR_BLACK;
 	m_Position = gccv::Normalscript;
+	m_Group = NULL;
 }
 
 gcpTextTool::~gcpTextTool ()
@@ -95,8 +102,8 @@ bool gcpTextTool::OnClicked ()
 			(static_cast <gccv::Item *> (m_Active) != dynamic_cast <gccv::ItemClient *> (m_pObject)->GetItem ())))
 		Unselect ();
 	bool create = false;
+	gcp::Theme *pTheme = m_pView->GetDoc ()->GetTheme ();
 	if (!m_pObject) {
-		gcp::Theme *pTheme = m_pView->GetDoc ()->GetTheme ();
 		gcp::Text *text = new gcp::Text(m_x0 / pTheme->GetZoomFactor (), m_y0 / pTheme->GetZoomFactor ());
 		m_pView->GetDoc ()->AddObject (text);
 		m_pView->GetDoc ()->AbortOperation ();
@@ -120,7 +127,11 @@ bool gcpTextTool::OnClicked ()
 			BuildTagsList ();
 		else
 			UpdateTagsList ();
+		m_Group = m_pObject->GetGroup ();
+		if (m_pView->GetDoc ()->GetCurrentOperation () == NULL && m_Group)
+			m_GroupNode = m_Group->Save (gcp::pXmlDoc);
 	}
+	m_Size = pTheme->GetTextFontSize () / PANGO_SCALE;
 	return true;
 }
 
@@ -238,7 +249,8 @@ bool gcpTextTool::OnKeyPress (GdkEventKey* event)
 
 bool gcpTextTool::OnKeyRelease (GdkEventKey* event)
 {
-	m_Active->OnKeyReleased (event);
+	if (m_Active)
+		m_Active->OnKeyReleased (event);
 	return true;
 }
 
@@ -271,59 +283,97 @@ bool gcpTextTool::Unselect ()
 	char const *text = m_Active->GetText ();
 	m_Active = NULL;
 	while (!m_UndoList.empty ()) {
-		xmlFree(m_UndoList.front ());
+		xmlUnlinkNode (m_UndoList.front ());
+		xmlFreeNode (m_UndoList.front ());
 		m_UndoList.pop_front ();
 	}
 	while (!m_RedoList.empty ()) {
-		xmlFree(m_RedoList.front ());
+		xmlUnlinkNode (m_RedoList.front ());
+		xmlFreeNode (m_RedoList.front ());
 		m_RedoList.pop_front ();
 	}
 	xmlBufferPtr initbuf = xmlBufferCreate ();
 	xmlBufferPtr endbuf = xmlBufferCreate ();
 	xmlNodeDump (initbuf, m_pApp->GetXmlDoc (), m_InitNode, 0, 0);
 	xmlNodeDump (endbuf, m_pApp->GetXmlDoc (), m_CurNode, 0, 0);
-	if (strcmp ((char*) initbuf->content, (char*) endbuf->content))
-	{
-		char* initval = (char*) xmlNodeGetContent (m_InitNode);
-		char* endval = (char*) xmlNodeGetContent (m_CurNode);
-		gcp::Operation *pOp = NULL;
-		if ((initval && strlen (initval))) {
-			if (endval && strlen (endval)) {
+	gcp::Operation *pOp = NULL;
+	if (strcmp ((char*) initbuf->content, (char*) endbuf->content)) {
+		if (m_Group) {
+			pOp = m_pView->GetDoc ()->GetCurrentOperation ();
+			if (!pOp) {
 				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_MODIFY_OPERATION);
-				pOp->AddNode (m_InitNode, 0);
-				pOp->AddNode (m_CurNode, 1);
-				m_CurNode = m_InitNode = NULL;
-			} else {
-				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_DELETE_OPERATION);
-				pOp->AddNode (m_InitNode);
-				m_InitNode = NULL;
+				pOp->AddNode (m_GroupNode, 0);
 			}
-		} else if (endval && strlen (endval)) {
-			pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_ADD_OPERATION);
-			pOp->AddNode (m_CurNode);
-			m_CurNode = NULL;
+			pOp->AddNode (m_Group->Save (gcp::pXmlDoc), 1);
+			m_Group = NULL;
+			m_GroupNode = NULL;
+		} else {
+			char* endval = (char*) xmlNodeGetContent (m_CurNode);
+			char* initval = (char*) xmlNodeGetContent (m_InitNode);
+			gcp::Fragment *fragment = dynamic_cast <gcp::Fragment *> (pObj);
+			map<Atom*, Bond*>::iterator i;
+			gcp::Bond *pBond = (fragment)? reinterpret_cast <gcp::Bond *> (fragment->GetAtom ()->GetFirstBond (i)): NULL;
+			if ((initval && strlen (initval))) {
+				if (endval && strlen (endval)) {
+					pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_MODIFY_OPERATION);
+					pOp->AddNode (m_InitNode, 0);
+					pOp->AddNode (m_CurNode, 1);
+					if (pBond) {
+						pOp->AddObject (pBond, 0);
+						pOp->AddObject (pBond, 1);
+					}
+					m_CurNode = m_InitNode = NULL;
+				} else {
+					pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_DELETE_OPERATION);
+					pOp->AddNode (m_InitNode);
+					if (pBond)
+						pOp->AddObject (pBond);
+					m_InitNode = NULL;
+				}
+			} else if (endval && strlen (endval)) {
+				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_ADD_OPERATION);
+				pOp->AddNode (m_CurNode);
+				m_CurNode = NULL;
+			}
+			if (initval)
+				xmlFree (initval);
+			if (endval)
+				xmlFree (endval);
 		}
-		if (initval)
-			xmlFree (initval);
-		if (endval)
-			xmlFree (endval);
 		if (pOp)
 			m_pView->GetDoc ()->PushOperation (pOp, m_bUndo);
 		m_bUndo = true;
+	} else {
+		if (m_Group) {
+			if (m_GroupNode) {
+				xmlFree (m_GroupNode);
+				m_GroupNode = NULL;
+			}
+			m_Group = NULL;
+		}
+		pOp = m_pView->GetDoc ()->GetCurrentOperation ();
+		if (pOp)
+			m_pView->GetDoc ()->AbortOperation ();
 	}
 	xmlBufferFree (initbuf);
 	xmlBufferFree (endbuf);
-	if (m_CurNode)
-		xmlFree (m_CurNode);
-	if (m_InitNode)
-		xmlFree (m_InitNode);
+	if (m_CurNode) {
+ 		xmlUnlinkNode (m_CurNode);
+		xmlFreeNode (m_CurNode);
+	}
+	if (m_InitNode) {
+		xmlUnlinkNode (m_InitNode);
+		xmlFreeNode (m_InitNode);
+	}
 	m_CurNode = m_InitNode = NULL;
 	if (!*text) {
 		Object* pMol = pObj->GetMolecule ();	//if pObj is a fragment
 		if (pMol)
 			pObj = pMol;
+		gcu::Object *parent = pObj->GetParent ();
 		m_pView->GetDoc ()->Remove (pObj);
 		m_pView->GetDoc ()->AbortOperation ();
+		parent->EmitSignal (gcp::OnChangedSignal);
 	}
 	m_pView->GetDoc ()->FinishOperation ();
 	m_pView->GetDoc ()->GetWindow ()->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", m_pView->GetDoc ()->HasChildren ());
@@ -394,20 +444,6 @@ bool gcpTextTool::PasteSelection (GtkClipboard *clipboard)
 	return true;
 }
 
-struct FragState {
-	PangoAttrList *l;
-	unsigned start;
-};
-
-static bool filter_fragment (PangoAttribute *attr, struct FragState *s)
-{
-	PangoAttribute *new_attr = pango_attribute_copy (attr);
-	new_attr->start_index += s->start;
-	new_attr->end_index += s->start;
-	pango_attr_list_change (s->l, new_attr);
-	return false;
-}
-
 bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *selection_data, G_GNUC_UNUSED int type)
 {
 	if (!m_Active)
@@ -433,18 +469,51 @@ bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *selectio
 				xmlFreeDoc (xml);
 				return true; // otherwise, we'd call OnChanged (true) twice.
 			} else if (!strcmp((char*)node->name, "fragment")) {
-// FIXME
-/*				gcp::Fragment* fragment = new gcp::Fragment ();
+				gcp::Fragment* fragment = new gcp::Fragment ();
 				gcp::Document *pDoc = m_pView->GetDoc ();
-				gcp::Theme *pTheme = pDoc->GetTheme ();
 				pDoc->AddChild (fragment);
 				fragment->Load (node);
 				string buf = fragment->GetBuffer ();
-				PangoAttrList *l = pango_attr_list_new ();
-				PangoAttribute *attr = pango_attr_family_new (pTheme->GetFontFamily ());
-				pango_attr_list_insert (l, attr);
-				attr = pango_attr_size_new (pTheme->GetFontSize ());
-				pango_attr_list_insert (l, attr);*/
+				m_Active->ReplaceText (buf, static_cast <int> (start), start - end);
+				gccv::TextTagList tags = fragment->GetTagList ();
+				gccv::TextTagList::iterator it, end = tags.end ();
+				gccv::TextTag *tag;
+				for (it = tags.begin (); it != end; it++) {
+					switch ((*it)->GetTag ()) {
+					case gccv::Family:
+					case gccv::Size:
+					case gccv::Style:
+					case gccv::Weight:
+					case gccv::Variant:
+					case gccv::Stretch:
+					case gccv::Underline:
+					case gccv::Overline:
+					case gccv::Strikethrough:
+					case gccv::Foreground:
+					case gccv::Background:
+					case gccv::Rise:
+					case gccv::NewLine:
+						tag = (*it)->Duplicate ();
+						break;
+					default: {
+						gccv::PositionTextTag *ptag = dynamic_cast <gccv::PositionTextTag *> (*it);
+						if (!ptag)
+							continue; // should not happen
+						bool stacked;
+						double size;
+						if (!ptag)
+							continue; // should not happen
+						gccv::TextPosition position = ptag->GetPosition (stacked, size);
+						tag = new gccv::PositionTextTag (position, stacked, size);
+						break;
+					}
+					}
+					tag->SetStartIndex ((*it)->GetStartIndex () + start);
+					tag->SetEndIndex ((*it)->GetEndIndex () + start);
+					m_Active->InsertTextTag (tag);
+				}
+				tags.clear ();
+				delete fragment;
 			} else {
 				xmlFreeDoc (xml);
 				return false;
@@ -482,7 +551,10 @@ bool gcpTextTool::OnUndo ()
 		{
 			if (!m_RedoList.empty())
 			{
-				if (m_CurNode) xmlFree(m_CurNode);
+				if (m_CurNode) {
+					xmlUnlinkNode (m_CurNode);
+					xmlFreeNode (m_CurNode);
+				}
 				m_CurNode = m_RedoList.back();
 				m_RedoList.pop_back();
 			}
@@ -526,7 +598,8 @@ void gcpTextTool::PushNode (xmlNodePtr node)
 {
 	gcp::Window *pWin = m_pView->GetDoc ()->GetWindow ();
 	while (!m_RedoList.empty ()) {
-		xmlFree (m_RedoList.front ());
+		xmlUnlinkNode (m_RedoList.front ());
+		xmlFreeNode (m_RedoList.front ());
 		m_RedoList.pop_front ();
 		pWin->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
 	}
@@ -555,7 +628,6 @@ void gcpTextTool::OnGetData (GtkClipboard *clipboard, GtkSelectionData *selectio
 		xmlDocDumpFormatMemory (pDoc, &gcp::ClipboardData, &size, info);
 		gtk_selection_data_set (selection_data, gdk_atom_intern (GCHEMPAINT_ATOM_NAME, FALSE), 8,  (const guchar*) gcp::ClipboardData, size);
 	}
-	gcp::cleared = false;
 	if (clipboard == gtk_clipboard_get (GDK_SELECTION_CLIPBOARD))
 			m_pView->GetDoc ()->GetWindow ()->ActivateActionWidget ("/MainMenu/EditMenu/Paste", true);
 }
@@ -627,7 +699,7 @@ void gcpTextTool::UpdateTagsList ()
 	}
 	// select the found face
 	GtkTreeIter iter;
-	char const *buf;
+	char *buf;
 	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (m_FamilyList), &iter);
 	do {
 		gtk_tree_model_get (GTK_TREE_MODEL (m_FamilyList), &iter, 0, &buf, -1);
@@ -640,8 +712,10 @@ void gcpTextTool::UpdateTagsList ()
 			g_signal_handler_unblock (m_FaceSel, m_FaceSignal);
 			g_signal_handler_unblock (m_FamilySel, m_FamilySignal);
 			gtk_tree_path_free (path);
+			g_free (buf);
 			break;
 		}
+		g_free (buf);
 	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (m_FamilyList), &iter));
 	// Update other widgets
 	SetSizeFull (true, false);
@@ -792,14 +866,19 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 	string name;
 	bool default_found = false;
 	for (i = 0; i < nb; i++) {
-		PangoFontFace **faces;
-		int *sizes, n;
+		PangoFontFace **faces = NULL;
+		int *sizes = NULL, n;
 		pango_font_family_list_faces (families[i], &faces, &n);
-		if (n <= 0)
+		if (n <= 0) {
+			g_free (faces);
 			continue;
+		}
 		pango_font_face_list_sizes (faces[0], &sizes, &n);
-		if (n > 0) // Do not use bitmap fonts
+		if (n > 0) {// Do not use bitmap fonts
+			g_free (faces);
+			g_free (sizes);
 			continue;
+		}
 		name = pango_font_family_get_name (families[i]);
 		m_Families[name] = (PangoFontFamily*) g_object_ref (families[i]);
 		gtk_list_store_append (m_FamilyList, &iter);
@@ -810,7 +889,10 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 			selected = iter;
 			default_found = true;
 		}
+		g_free (faces);
+		g_free (sizes);
 	}
+	g_free (families);
 	if (!default_found) {
 		if	(!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (m_FamilyList), &iter))
 			return NULL;
@@ -853,6 +935,7 @@ void gcpTextTool::OnSelectFamily (GtkTreeSelection *selection)
 		return;
 	gtk_tree_model_get (model, &iter, 0, &name, -1);
 	m_FamilyName = name;
+	g_free (const_cast <char *> (name));
 	PangoFontFamily *family = m_Families[m_FamilyName];
 	PangoFontFace **faces;
 	int i, besti, nb;
@@ -896,6 +979,7 @@ void gcpTextTool::OnSelectFamily (GtkTreeSelection *selection)
 		// TODO: write this code
 		pango_font_description_free (desc);
 	}
+	g_free (faces);
 	g_signal_handler_unblock (m_FaceSel, m_FaceSignal);
 	GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (m_FaceList), &selected);
 	if (path) {
@@ -919,11 +1003,12 @@ void gcpTextTool::OnSelectFace (GtkTreeSelection *selection)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	char const *name;
+	char *name;
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
 		return;
 	gtk_tree_model_get (model, &iter, 0, &name, -1);
 	PangoFontFace *face = m_Faces[name];
+	g_free (name);
 	PangoFontDescription *desc = pango_font_face_describe (face);
 	m_Style = pango_font_description_get_style (desc);
 	m_Weight = pango_font_description_get_weight (desc);
